@@ -1480,6 +1480,7 @@ panel_profile_delete_toplevel_objects (const char        *toplevel_id,
 
 		if (parent_toplevel_id && !strcmp (toplevel_id, parent_toplevel_id)) {
 			g_free (id);
+			g_free (parent_toplevel_id);
 			continue;
 		}
 
@@ -2419,9 +2420,9 @@ panel_profile_get_enable_program_list (void)
 }
 
 gboolean
-panel_profile_get_enable_autocomplete (void)
+panel_profile_get_enable_autocompletion (void)
 {
-	return get_program_listing_setting ("enable_autocomplete");
+	return get_program_listing_setting ("enable_autocompletion");
 }
 
 void
